/*
 *
 *
 *
 *
 *   DO  NOT  EDIT  THIS  FILE !!
 *
 *   AUTOMATICALLY  GENERATED  DURING  BUILD
 *   FORM THE PUBLIC HEADER FILE TEMPLATES
 *
 *
 *
 *
 *
 *
*/

#ifndef LIBBB5_H
#define LIBBB5_H

#ifdef __cplusplus
extern "C" {
#endif

#include <stdlib.h>
#include <stdint.h>


/******************************************************
               CCC services header START
*******************************************************/

/** CCC CS type format */
typedef uint8_t _bb5_ccc_cs_type_t[136];
/** CCC ID format */
typedef uint8_t _bb5_ccc_id_t[8];

/** Generic fields read from the CCC by bb5_ccc_info_read() */
struct bb5_ccc_res_info_read
{
  _bb5_ccc_id_t ccc_id; 	/**< ID of the CCC */
  uint32_t ccc_version; 		/**< Version of the CCC */
  uint8_t flags; 			/**< Flags for the CCC, e.g. if extension data is used or not */
  uint8_t spare0[3];			/**< Padding */
  uint32_t spare1[4]; 			/**< Padding */
  uint32_t extension_length; 		/**< Length of the extension data in CCC */
  uint32_t core_sw_timestamp; 		/**< Timestamp for core software image */
  uint32_t variant_sw_timestamp; 	/**< Timestamp for the variant software image */
  _bb5_ccc_id_t cs_type;	 	/**< CS type information */
};


/**
 * @brief Flashes CCC to CAL.
 * @details Flashes given Common Configuration Certificate to the CAL
 * area. Used in the production in the labelling phase.
 * Does integrity and authenticity check for the data.
 *
 *
 *  @credential ibbb5-secbins::SEE_CCCWrite    Required for all use cases
 *  @credential GRP::cal                       Required for all use cases
 *
 * 	@param[in]	buf		The certificate data buffer.
 *                                      Must not be NULL.
 *
 * 	@param[in]	len		Length of the data buffer.
 *
 *	@return		0		Success.
 *	    		<0		Generic error.
 *
*/
extern int bb5_ccc_cert_flash(const uint8_t * buf, size_t len);

/**
 * @brief Flashes HWC to CAL.
 * @details Flashes given Hardware Configuration Certificate to the
 * CAL area. Used in the production in the labelling phase.
 * Does integrity and authenticity check for the data.
 *
 *
 *  @credential libbb5-secbins::SEE_HWCWrite    Required for all use cases
 *  @credential GRP::cal                        Required for all use cases
 *
 * 	@param[in]	buf		The certificate data buffer.
 *                                      Must not be NULL.
 *
 * 	@param[in]	len		Length of the data buffer.
 *
 *	@return		0		Success.
 *	    		<0		Generic error.
 *
*/
extern int bb5_hwc_cert_flash(const uint8_t * buf, size_t len);

/**
 * @brief Write CCC to CAL.
 * @details Writes the Common Configuration Certificate to the CAL
 * area. Does not do any validity or integrity checks for the data.
 *
 * DEPRECATED! Use bb5_ccc_cert_flash() instead!
 *
 *
 * 	@param[in]	buf	The buffer containing the certificate data.
 *
 * 	@param[in]	len	The length of the certificate data to write
 *
 *	@return		0	Success.
 *	    		-1	Buffer or CAL block name is a NULL ptr
 *	    		-2	CAL init failed
 *	    		-3	CAL block write failed
 *
*/
extern ssize_t bb5_ccc_cert_write(const uint8_t * buf, size_t len) __attribute__ ((deprecated));

/**
 * @brief Erases CCC from CAL.
 * @details Erases the Common Configuration Certificate from the CAL
 * area.
 *
 *  @credential GRP::cal    Required for all use cases
 *
 *	@return		0	Success.
 *	    		<0	Generic error.
 *
*/
extern int bb5_ccc_cert_erase();

/**
 * @brief Erases HWC from CAL.
 * @details Erases the HardWare Certificate from the CAL area.
 *
 *  @credential GRP::cal    Required for all use cases
 *
 *	@return		0	Success.
 *	    		<0	Generic error.
 *
*/
extern int bb5_hwc_cert_erase();

/**
 * @brief Reads CCC from CAL.
 * @details Reads the Common Configuration Certificate from the CAL
 * area. Does not do any validity or integrity checks for the data.
 * The call will allocate the result buffer.
 *
 *  @credential GRP::cal    Required for all use cases
 *
 * 	@param[out]	buf	Pointer to the result buffer allocated
 *				by libbb5 that contains the data from CAL.
 *                              Use bb5_dealloc_result() to deallocate it.
 *
 * 	@param[out]	len	The length of the read data set by the library.
 *
 *	@return		0	Success.
 *	    		-1	Buffer, length or CAL block name
 *				is a NULL pointer
 *	    		-2	CAL init failed
 *	    		-3	CAL block read failed
 *	    		-4	CAL block not found (was empty)
 *
*/
extern int bb5_ccc_cert_read(uint8_t **buf, size_t * len);


/**
 * @brief Writes HWC to CAL.
 * @details Writes the HardWare Certificate to the CAL area.
 * Does not do any validity or integrity checks for the data.
 *
 * DEPRECATED! Use bb5_hwc_cert_flash() instead!
 *
 *
 * 	@param[in]	buf	The buffer containing the certificate data.
 *
 * 	@param[in]	len	The length of the certificate data to write
 *
 *	@return		0	Success.
 *	    		-1	Buffer or CAL block name is a NULL ptr
 *	    		-2	CAL init failed
 *	    		-3	CAL block write failed
 *
*/
extern int bb5_hwc_cert_write(const uint8_t * buf, size_t len) __attribute__ ((deprecated));

/**
 * @brief Reads HWC from CAL.
 * @details Reads the HardWare Certificate from the CAL area.
 * Does not do any validity or integrity checks for the data.
 * The call will allocate the result buffer.
 *
 *  @credential GRP::cal    Required for all use cases
 *
 * 	@param[out]	buf	Pointer to the result buffer allocated
 *				by libbb5 that contains the data from CAL.
 *                              Use bb5_dealloc_result() to deallocate it.
 *
 * 	@param[out]	len	The length of the read data set by the library.
 *
 *	@return		0	Success.
 *	    		-1	Buffer, length or CAL block name
 *				is a NULL pointer
 *	    		-2	CAL init failed
 *	    		-3	CAL block read failed
 *	    		-4	CAL block not found (was empty)
 *
*/
extern int bb5_hwc_cert_read(uint8_t **buf,
				 size_t * len);


/**
 * @brief Imports CCC.
 * @details Imports the Common Configuration Certificate (CCC) into the
 * secure environment. Must be called only after HWC import has been
 * done successfully
 *
 *  @credential GRP::cal    Required for all use cases
 *
 * 	@param[in]	ccc_ptr		Pointer to the certificate data to use.
 *					If NULL, will use certificate data
 *					from CAL.
 *
 * 	@param[in]	ccc_len		The length of the certificate data.
 *
 *	@return		0		Success.
 *	    		<0		Generic error.
*/
extern int bb5_ccc_init(const uint8_t * ccc_ptr,
                            size_t ccc_len);


/**
 * @brief Imports HWC.
 * @details Imports the HardWare Certificate (HWC) into the secure
 * environment. Must be called and executed successfully before
 * importing CCC.
 *
 *  @credential GRP::cal    Required for all use cases
 *
 * 	@param[in]	hwc_ptr		Pointer to the certificate data to use.
 *					If NULL, will use certificate data
 *					from CAL.
 *
 * 	@param[in]	hwc_len		The length of the certificate data.
 *
 *	@return		0		Success.
 *	    		<0		Generic error.
*/
extern int bb5_ccc_hwc_init(const uint8_t * hwc_ptr, size_t hwc_len);


/**
 * @brief Reads data from CCC.
 * @details Reads fields from Common Configuration Certificate. Does
 * not include extension data.
 *
 * CCC must be imported before any data can be read from the certificate.
 *
 *  @credential GRP::cal    Required for all use cases
 *
 * 	@param[out]	ccc_info	Result buffer for the CCC data.
 *
 *	@return		0		Success.
 *	    		<0		Generic error.
*/
extern int bb5_ccc_info_read(struct bb5_ccc_res_info_read * ccc_info);

/**
 * @brief Reads extension data from CCC.
 * @details Reads the extension data fields from the Common
 * Configuration Certificate according to a given extension type.
 * CCC must be imported before any data can be read from the certificate.
 * The call allocates the buffer for the extension data.
 *
 *  @credential GRP::cal    Required for all use cases
 *
 * 	@param[in]	ext_type	The extension type.
 *
 * 	@param[out]	ext_buf		Result buffer allocated by the library
 *					that contains the extension data.
 *                                      Use bb5_dealloc_result() to deallocate it.
 *
 *	@return		>=0		Success, the length of the extension
 *	    				data.
 *	    		<0		Generic error.
*/
extern ssize_t bb5_ccc_extdata_read(uint32_t ext_type, uint8_t **ext_buf);

/******************************************************
               CCC services header END
*******************************************************/


/******************************************************
               Common services header START
*******************************************************/

#include <stdlib.h>
#include <stdint.h>

#define BB5_INFO_NAME_MAXLEN       50

/** Hash (SHA-1) buffer used in libbb5 */
typedef uint8_t bb5_hash_sha1_t[20];

/** Hash (SHA-2) buffer used in libbb5 */
typedef uint8_t bb5_hash_sha2_t[32];

typedef uint8_t bb5_hmac_t[20];

typedef uint8_t bb5_publicid_t[20];

typedef uint32_t bb5_debug_control_t[3];

typedef char bb5_labelstr_t[BB5_INFO_NAME_MAXLEN+1];



/**
 * @brief Initializes libbb5.
 * @details Opens the /dev/omap_sec device interface to the kernel.
 * Needs to be called before all other libbb5 calls.
 *
 * 	@param[in]	pa_name		Prefix of PA/FMT/PAPUB binaries to
 *					load from /lib/firmware.
 *					If NULL, uses the default prefix
 *					defined	by the omap_sec kernel driver.
 *
 *	@return		0		Success.
 *	    		-1		Unable to open /dev/omap_sec
 *	    		-2		Unable to load given PA/FMT/PAPUB
 *					binaries.
*/
extern int bb5_open(const uint8_t *pa_name);

/**
 * @brief Performs clean-up operations.
 * @details Closes the device interface to kernel.
 * Needs to be called after all other libbb5 calls are done.
 *
 *	@return		0		Success.
*/
extern int bb5_close(void);


/**
 * @brief Reads error codes.
 * @details Returns error codes from various components involved in the different
 * stages of execution of the previous libbb5 call.
 *
 * 	@param[out]	err_lib		The errno value set by the kernel
 *					driver,	or library itself.
 *					If NULL, does not set this error
 *					code
 *
 * 	@param[out]	err_rom		The error code set by the ROM API.
 *					If NULL, does not set this error
 *					code
 *
 * 	@param[out]	err_pa		The error code set by the PA.
 *					If NULL, does not set this error
 *					code
 *
 *	@return		0		Success, all error codes set by
 *					the corresponding components.
 *	    		-1		Success, some error codes are in
 *					uninitialized state.
 *					E.g. if the execution did not go
 *					through all of the components.
*/
extern int bb5_read_error(uint32_t * err_lib,
			      uint32_t * err_rom,
			      uint32_t * err_pa);


/**
 * @brief Displays error information to stdout.
 * @details Displays information on the error codes for the previous
 * libbb5 call. Output will be printed to standard output.
 *
 * 	@param[in]	op_name		String representation of the operation
 *					that failed. Will be included into the
 *					printout.
 *					Needs to be a NULL-terminated string.
 *					Think of perror().
 *	@return		0		Success.
*/
extern int bb5_display_error(const char * op_name);


/**
 * @brief Sets the application ID.
 * @details Sets the application ID to be used with subsequent
 * application ID dependent libbb5 calls.
 *
 * 	@param[in]	new_appl_id	The new application ID to use
 *					Needs to be a NULL-terminated string.
 *					Maximum size 1024 bytes.
 *
 *	@return		0		Success.
 *	    		-1		New application ID is invalid. NULL-
 *					pointer or too long string.
 *	    		-2		Not enough memory for new internal
 *					application ID buffer.
*/
extern int  bb5_set_programid(const uint8_t * new_appl_id);


/**
 * @brief Deallocates a result buffer.
 * @details Deallocates a result buffer previously allocated by a
 * library call.
 *
 * 	@param[in]	buf_ptr		Result buffer.
 *
*/
extern void bb5_dealloc_result(void *buf_ptr);

/**
 * @brief Enables debug prints.
 * @details Enables debug prints.
 * Works only with a debug version of the library.
 *
*/
extern void bb5_debug_enable(void);

/**
 * @brief Disables debug prints.
 * @details Disables debug prints.
 * Works only with a debug version of the library.
 *
*/
extern void bb5_debug_disable(void);

/******************************************************
               Common services header START
*******************************************************/



/******************************************************
           Crypto services header START
*******************************************************/

#define SHA1_HASH_LEN   20
#define	AES_BLOCK_SIZE	16
#define	AES_KEY_SIZE	32

typedef struct _bb5_crypto_ctx bb5_crypto_ctx;

typedef bb5_crypto_ctx bb5_sha_ctx;
typedef bb5_crypto_ctx bb5_aes_ctx;

/* SHA1 API */
bb5_sha_ctx *bb5_sha1_init(void);
int bb5_sha1_update(bb5_sha_ctx *ctx, const void *data, unsigned long len);
int bb5_sha1_final(bb5_sha_ctx *ctx, unsigned char *md);

/* AES API */
bb5_aes_ctx *bb5_aes_cbc_init(const unsigned char *key, int keylen);
int bb5_aes_cleanup(bb5_aes_ctx *ctx);
int bb5_aes_crypt(bb5_aes_ctx *ctx, const unsigned char *in, unsigned char *out, int len, const unsigned char *iv, int enc);
int bb5_aes_encrypt(bb5_aes_ctx *ctx, const unsigned char *in, unsigned char *out, int len, const unsigned char *iv);
int bb5_aes_decrypt(bb5_aes_ctx *ctx, const unsigned char *in, unsigned char *out, int len, const unsigned char *iv);

/******************************************************
           Crypto services header END
*******************************************************/


/******************************************************
    Digital Battery Interface services header START
*******************************************************/

/** Number of DBI pairsets total */
#define BB5_DBI_PAIRSET_NUM             10
/** Number of challenge-response pairs in a given pairset */
#define BB5_DBI_PAIR_NUM                10

/** Length of challenge buffer */
#define BB5_DBI_CHALLENGE_LEN           6
/** Length of response buffer */
#define BB5_DBI_RESPONSE_LEN            3
/** Length of padding for AES required */
#define BB5_DBI_AES_PAD_LEN             2

/** Challenge buffer type */
typedef uint8_t bb5_dbi_chal_t[BB5_DBI_CHALLENGE_LEN];
/** Response buffer type */
typedef uint8_t bb5_dbi_resp_t[BB5_DBI_RESPONSE_LEN];

/** Syntax for challenge-response pair */
struct _bb5_dbi_pair
{
	uint8_t challenge[BB5_DBI_CHALLENGE_LEN]; // Challenge
	uint8_t response[BB5_DBI_RESPONSE_LEN]; // Response
};

/**
 * Syntax for a pairset, containing multiple challenge-response pairs,
 * identified by a DBI key index.
*/
struct _bb5_dbi_pairset
{
	uint8_t key_identifier; // Identifier for key used to generate callenge-response pairs
	struct _bb5_dbi_pair pair[BB5_DBI_PAIR_NUM]; // 10 challenge-response pair structures
};

/** Syntax for a keyset, containing multiple pairsets */
struct bb5_dbi_keysets_enc
{
	struct _bb5_dbi_pairset pair_set[BB5_DBI_PAIRSET_NUM];
	uint8_t padding[BB5_DBI_AES_PAD_LEN];
};

/**
 * Format of DBI data saved to CAL.
*/
struct bb5_dbiv1_cal_data
{
  uint32_t version;
  struct bb5_dbi_keysets_enc keysets;
  bb5_hmac_t hmac;
};

/**
 * Format of ASED data received from the production in labelling
*/
struct bb5_dbiv1_ased_data
{
  uint32_t version;
  uint32_t enc_len; /**< Length of the encrypted block */
  struct bb5_dbi_keysets_enc keysets;
  bb5_hmac_t hmac;
};

/**
 * @brief Flashes DBI data to CAL.
 * @details Flashes given DBI ASED data to the CAL area.
 * Used in the production in the labelling phase.
 * Does integrity and authenticity check for the data.
 *
 *
 *  @credential libbb5-secbins::SEE_DBIWrite    Required for all use cases
 *  @credential GRP::cal                        Required for all use cases
 *
 * 	@param[in]	ased_data	ASED data buffer
 *                                      Must not be NULL.
 *
 *
 *	@return		0		Success
 *	    		<0		Generic error
 *
*/
extern int bb5_dbi_data_flash(const struct bb5_dbiv1_ased_data *ased_data);


/**
 * @brief Writes DBI data to CAL.
 * @details Writes given DBI ASED data to the CAL area.
 * Does not do and validity or integrity checks for the data.
 *
 * DEPRECATED! Use bb5_dbi_data_flash() instead!
 *
 * 	@param[in]	cal_data	CAL Data buffer
 *                                      Must not be NULL.
 *
 *
 *	@return		0		Success
 *	    		<0		Generic error
 *
*/
extern int bb5_dbi_data_write(struct bb5_dbiv1_cal_data *cal_data) __attribute__ ((deprecated));


/**
 * @brief Reads DBI data from CAL.
 * @details Reads DBI data from the CAL area.
 * Does not do and validity or integrity checks for the data.
 *
 *  @credential GRP::cal    Required for all use cases
 *
 * 	@param[out]	cal_data	Result buffer for the CAL data
 *                                      Allocated by the library.
 *                                      Must not be NULL.
 *                                      Use bb5_dealloc_result() to deallocate it.
 *
 *	@return		0		Success
 *	    		<0		Generic error
 *
*/
extern int bb5_dbi_data_read(struct bb5_dbiv1_cal_data **cal_data);

/**
 * @brief Performs DBI challenge get.
 * @details Performs the challenge get phase of the Digital Batter
 * Interface challenge-response authentication.
 *
 *  @credential GRP::cal        Required for all use cases
 *
 * 	@param[in]	version         Version of the DBI data.
 *
 * 	@param[in]	sets_enc        Encrypted keysets
 *                                      Must not be NULL.
 *
 * 	@param[in]	hmac            HMAC over the version and keysets
 *                                      Must not be NULL.
 *
 * 	@param[in]	key_id          DBI key index to use
 *
 * 	@param[out]	challenge       Result buffer for the challenge
 *                                      Must not be NULL.
 *
 *	@return		0		Success
 *	    		<0		Generic error
 *
*/
extern int bb5_dbi_challenge_get(	uint32_t version,
					const struct bb5_dbi_keysets_enc * sets_enc,
					const bb5_hmac_t * hmac,
					uint8_t key_id,
					bb5_dbi_chal_t *challenge) __attribute__ ((deprecated));

/******************************************************
    Digital Battery Interface services header END
*******************************************************/



/******************************************************
    Data integrity verification services header START
*******************************************************/


/**
 * Defines different levels of protection for
 * different data types.
*/
enum bb5_div_type
{
        /**
          * Hash will be saved to non-volatile
          * secure storage. SuperDongle needs to be
          * authenticated or an appropriate RnD certificate
          * flashed into the device to use integrity verification
          * services for this data type.
        */
	BB5_DIV_TYPE_SECURE = 1,

        /**
          * Hash will be saved to non-volatile
          * secure storage.
        */
	BB5_DIV_TYPE_NORMAL = 2,

        /** Hash will be saved to secure RAM */
	BB5_DIV_TYPE_LIGHT = 3,

};


/**
 * @brief Saves data for integrity verification.
 * @details Calculates a hash of the given data identified uniquely with
 * an ID, and saves the hash to secure environment. Type of data given
 * determines how data is saved, and if SuperDongle needs to be
 * authenticated to save the hash.
 *
 *  @credential GRP::cal
 *
 * 	@param[in]	type		The type of data item.
 *
 * 	@param[in]	id		Unique ID for the data item.
 *
 * 	@param[in]	buf		The data buffer.
 *                                      Must not be NULL.
 *
 * 	@param[in]	len		The length of the data buffer.
 *
 *	@return		0		Success.
 *	    		<0		Generic error.
 *
*/
extern int bb5_div_id_update(enum bb5_div_type type,
                                 uint8_t id,
                                 const uint8_t *buf,
                                 size_t len);

/**
 * @brief Does integrity check for given data
 * @details Checks whether a given data item has changed since its hash
 * was saved to secure environment with a previous call to
 * bb5_div_id_update().
 *
 *  @credential GRP::cal
 *
 * 	@param[in]	type		The type of data item.
 *
 * 	@param[in]	id		Unique ID for the data item.
 *
 * 	@param[in]	buf		The data buffer.
 *                                      Must not be NULL.
 *
 * 	@param[in]	len		The length of the data buffer.
 *
 *	@return		0		Success, data has not changed.
 *	    		<0		Generic error, or data has changed.
 *
*/
extern int bb5_div_id_check(enum bb5_div_type type,
                                uint8_t id,
                                const uint8_t *buf,
                                size_t len);


/******************************************************
    Data integrity verification services header END
*******************************************************/



/******************************************************
        Device lock services header START
*******************************************************/

/*
* Public API for Device lock (dl_mc1) PA
*/

/** state enumerations for device lock */
enum bb5_devlock_state
{
  BB5_DEVLOCK_STATE_DISABLED = 0,
  BB5_DEVLOCK_STATE_ENABLED = 1
};

/** state enumerations for wiping */
enum bb5_wiping_state
{
  BB5_WIPING_NOT_NEEDED = 0,
  BB5_WIPING_NEEDED = 1
};

/** state enumerations for device lock password */
enum bb5_password_state
{
  BB5_PASSWORD_NOT_DEFINED = 0,
  BB5_PASSWORD_DEFINED = 1
};

/** method enumerations for wiping */
enum bb5_wiping_method
{
  BB5_NORMAL_WIPING  = 0,
  BB5_SECURE_WIPING = 1
};


/**
 * @brief Writes device lock settings to CAL.
 * @details Writes the device lock secure settings string to CAL area.
 *
 *  @credential GRP::cal
 *
 * 	@param[in]	length          Settings buffer length
 *
 * 	@param[in]	settings_buf    Settings buffer
 *
 *	@return		0               Success
 *	    		<0              Generic error
 *
 */
extern int bb5_devlock_set_sec_settings(size_t length, const uint8_t *settings_buf);

/**
 * @brief Reads device lock settings from CAL.
 * @details Reads the device lock secure settings string from CAL area.
 *
 *  @credential GRP::cal
 *
 * 	@param[out]	length          Settings buffer length
 *
 * 	@param[out]	settings_buf    Settings buffer
 *
 *	@return		0               Success
 *	    		<0              Generic error
 *
 */
extern int bb5_devlock_get_sec_settings(size_t *length, uint8_t **settings_buf);

/**
 * @brief Sets the device lock password.
 * @details Sets the device lock password.
 * Old password must be provided to be able to set a new one. However,
 * verification of the old password is skipped, if there were no password
 * defined earlier.
 *
 *  @credential libbb5-secbins::SEE_DeviceLockControl
 *  @credential GRP::cal
 *
 * 	@param[in]	old_password_length     buffer length for old passsword
 *
 * 	@param[in]	old_password            buffer for old password
 *
 * 	@param[in]	new_password_length     buffer length for new passsword
 *
 * 	@param[in]	new_password            buffer for new password
 *
 *	@return		0                       Success
 *	    		<0                      Generic error
 *
 */
extern int bb5_devlock_set_password(size_t old_password_length,
                                    const uint8_t *old_password,
                                    size_t new_password_length,
                                    const uint8_t *new_password);

/**
 * @brief Verifies the device lock password.
 * @details Verifies the device lock password.
 *
 *  @credential libbb5-secbins::SEE_DeviceLockControl
 *  @credential GRP::cal
 *
 * 	@param[in]	password_length     buffer length for passsword
 *
 * 	@param[in]	password            buffer for password
 *
 *	@return		0                   Success
 *	    		<0                  Generic error
 *
 */
extern int bb5_devlock_verify_password(size_t password_length,
                                       const uint8_t *password);

/**
 * @brief Sets device lock state.
 * @details Sets device lock state.
 * A correct password must be provided for changing device lock
 * state. If there were no password defined when enabling device
 * lock, given password will be set into use.
 *
 *  @credential libbb5-secbins::SEE_DeviceLockControl
 *  @credential GRP::cal
 *
 * 	@param[in]	password_length     buffer length for passsword
 *
 * 	@param[in]	password            buffer for password
 *
 * 	@param[in]	state               Device lock state
 *
 *	@return		0                   Success
 *	    		<0                  Generic error
 *
 */
extern int bb5_devlock_set_devlock_state(size_t password_length,
                                         const uint8_t *password,
                                         enum bb5_devlock_state state);

/**
 * @brief Reads device lock state.
 * @details Reads device lock state.
 *
 *  @credential libbb5-secbins::SEE_DeviceLockControl
 *  @credential GRP::cal
 *
 * 	@param[out]	state               Device lock state
 *
 *	@return		0                   Success
 *	    		<0                  Generic error
 *
 */
extern int bb5_devlock_get_devlock_state(enum bb5_devlock_state *state);

/**
 * @brief Sets user data wiping state.
 * @details Sets user data wiping state.
 *
 *  @credential GRP::cal
 *
 * 	@param[in]	state               Wiping state
 *
 *	@return		0                   Success
 *	    		<0                  Generic error
 *
 */
extern int bb5_devlock_set_wiping_state(enum bb5_wiping_state state);

/**
 * @brief Reads user data wiping state.
 * @details Reads user data wiping state.
 *
 *  @credential libbb5-secbins::SEE_DeviceLockControl
 *  @credential GRP::cal
 *
 * 	@param[out]	state               Wiping state
 *
 *	@return		0                   Success
 *	    		<0                  Generic error
 *
 */
extern int bb5_devlock_get_wiping_state(enum bb5_wiping_state *state);

/**
 * @brief Resets device lock state.
 * @details This function resets device lock to default state.
 * User data must be erased first before calling this function.
 *
 *  @credential GRP::cal
 *
 *	@return		0                   Success
 *	    		<0                  Generic error
 *
 */
extern int bb5_devlock_reset();

/**
 * @brief Reads password state.
 * @details Reads password state.
 *
 *  @credential libbb5-secbins::SEE_DeviceLockControl
 *  @credential GRP::cal
 *
 * 	@param[out]	state               Password state
 *
 *	@return		0                   Success
 *	    		<0                  Generic error
 *
 */
int bb5_devlock_get_password_state(enum bb5_password_state *state);

/**
 * @brief Sets user data wiping method.
 * @details Sets user data wiping method.
 *
 *  @credential libbb5-secbins::SEE_DeviceLockControl
 *  @credential GRP::cal
 *
 * 	@param[in]	method              Wiping method
 *
 *	@return		0                   Success
 *	    		<0                  Generic error
 *
 */
extern int bb5_devlock_set_wiping_method(enum bb5_wiping_method method);

/**
 * @brief Reads user data wiping method.
 * @details Reads user data wiping method.
 *
 *  @credential GRP::cal
 *
 * 	@param[out]	method              Wiping method
 *
 *	@return		0                   Success
 *	    		<0                  Generic error
 *
 */
extern int bb5_devlock_get_wiping_method(enum bb5_wiping_method *method);



/******************************************************
        Device lock services header END
*******************************************************/


/******************************************************
              ESN services header START
*******************************************************/

typedef uint8_t bb5_esn_label_t[8];
typedef uint8_t bb5_esn_data_t[32];
typedef uint8_t bb5_imei_raw_t[8];
typedef uint8_t bb5_imei_t[15];
typedef uint8_t bb5_imei_bcd_t[8]; // __attribute__ ((deprecated));


/**
 * @brief Flashes NPC to CAL.
 * @details Flashes given Nokia Public Certificate to the CAL area.
 * Used in the production in the labelling phase.
 * Does integrity and authenticity check for the data.
 *
 *  @credential libbb5-secbins::SEE_NPCWrite    Required for all use cases
 *  @credential GRP::cal                        Required for all use cases
 *
 * 	@param[in]	buf		The certificate data buffer.
 *                                      Must not be NULL.
 *
 * 	@param[in]	len		Length of the data buffer.
 *
 *	@return		0		Success.
 *	    		<0		Generic error.
 *
*/
extern int bb5_npc_cert_flash(const uint8_t * buf, size_t len);

/**
 * @brief Erases NPC from CAL.
 * @details Erases the Nokia Public Certificate from the CAL area.
 *
 *  @credential GRP::cal                        Required for all use cases
 *
 *	@return		0	Success.
 *	    		<0	Generic error.
 *
*/
extern int bb5_npc_cert_erase();


/**
 * @brief Writes NPC to CAL.
 * @details Writes the Nokia Public Certificate to the CAL area.
 * Does not do any validity or integrity checks for the data.
 *
 * DEPRECATED! Use bb5_npc_cert_flash() instead!
 *
 *
 * 	@param[in]	buf		The certificate data buffer.
 *                                      Must not be NULL.
 *
 * 	@param[in]	len		Length of the data buffer.
 *
 *	@return		0		Success.
 *	    		<0		Generic error.
 *
*/
extern int bb5_npc_cert_write(const uint8_t * buf, size_t len) __attribute__ ((deprecated));

/**
 * @brief Reads NPC from CAL.
 * @details Reads the Nokia Public Certificate from the CAL area.
 * Does not do any validity or integrity checks for the data.
 *
 *  @credential GRP::cal                        Required for all use cases
 *
 * 	@param[out]	buf		The certificate data buffer.
 *                                      The library will allocate the buffer.
 *                                      Must not be NULL.
 *
 * 	@param[out]	len		Length of the data buffer, set
 *                                      by the library.
 *                                      Must not be NULL.
 *
 *	@return		0		Success.
 *	    		<0		Generic error.
 *
*/
extern int bb5_npc_cert_read(uint8_t **buf, size_t *len);

/**
 * @brief Imports NPC.
 * @details Imports the Nokia Public Certificate to the secure
 * environment. Does integrity and authenticity check for the
 * certificate data.
 *
 *  @credential GRP::cal                        Required for all use cases
 *
 * 	@param[in]	buf		The certificate data buffer.
 *                                      If NULL, reads data from CAL
 *                                      and requires root privileges
 *                                      because of dependency to libcal.
 *
 * 	@param[in]	len		Length of the data buffer.
 *
 *	@return		0		Success.
 *	    		<0		Generic error.
 *
*/
extern int bb5_esn_init(const uint8_t * npc_ptr, size_t npc_len);


/**
 * @brief Reads raw ESNs from NPC.
 * @details Retrieves a given Electronic Serial Number from the Nokia
 * Public Certificate in raw format.
 * Does integrity check for the data.
 *
 * 	@param[in]	esn_label	Label for the ESN to retrieve.
 *                                      Must not be NULL pointer.
 *
 * 	@param[out]	number		Result buffer for the ESN value.
 *                                      Must be allocated by caller.
 *                                      Must not be NULL pointer.
 *
 *	@return		0		Success.
 *	    		<0		Generic error.
 *
*/
extern int bb5_esn_read(const bb5_esn_label_t *esn_label, bb5_esn_data_t *number);

/**
 * @brief Reads IMEI from NPC in raw.
 * @details Retrieves IMEI from the Nokia Public Certificate in raw
 * format. The format is currently BCD.
 * Does integrity check for the data.
 *
 * 	@param[out]	buf	        Result buffer for the IMEI.
 *                                      Allocated by the caller.
 *                                      Must not be NULL pointer.
 *
 *	@return		0		Success.
 *	    		<0		Generic error.
 *
*/
extern int bb5_esn_imei_read_raw(bb5_imei_raw_t *buf);

/**
 * @brief Reads IMEI from device in raw.
 * @details Retrieves IMEI from PPA (if feature enabled) in raw format.
 * As a backup uses method bb5_esn_imei_read_raw() when FAST IMEI
 * feature is not enabled. The format is currently BCD.
 * Does integrity check for the data.
 *
 * 	@param[out]	buf	        Result buffer for the IMEI.
 *                                      Allocated by the caller.
 *                                      Must not be NULL pointer.
 *
 *	@return		0		Success.
 *	    		<0		Generic error.
 *
*/
extern int bb5_imei_read_raw(bb5_imei_raw_t *buf);

/**
 * @brief Reads IMEI from NPC in raw.
 * @details Retrieves IMEI from the Nokia Public Certificate in raw BCD
 * format.
 * Does integrity check for the data.
 *
 * DEPRECATED. Use bb5_imei_read_raw() instead.
 *
 *
 *
 * 	@param[out]	buf	        Result buffer for the IMEI.
 *                                      Allocated by the caller.
 *                                      Must not be NULL pointer.
 *
 *	@return		0		Success.
 *	    		<0		Generic error.
 *
*/
extern int bb5_imei_read_bcd(bb5_imei_bcd_t *buf) __attribute__ ((deprecated));

/**
 * @brief Reads IMEI from NPC.
 * @details Retrieves IMEI from the Nokia Public Certificate in
 * human-readable format.
 * Does integrity check for the data.
 *
 * 	@param[out]	buf	        Result buffer for the IMEI.
 *                                      Allocated by the caller.
 *                                      Must not be NULL pointer.
 *
 *	@return		0		Success.
 *	    		<0		Generic error.
 *
*/
extern int bb5_imei_read(bb5_imei_t *imei);

/******************************************************
              ESN services header END
*******************************************************/


/******************************************************
              FPROT services header START
*******************************************************/

enum bb5_fprot_mac_result {
        BB5_FPROT_MAC_INVALID = 0,
        BB5_FPROT_MAC_VALID_TRUSTED,
        BB5_FPROT_MAC_SPARE,
        BB5_FPROT_MAC_VALID_UNTRUSTED,
};

/**
 * @brief Encrypt data.
 * @details Encrypts data with a client-/application- and
 * device-specific key.
 *
 * 	@param[in]	msg		Data.
 *
 * 	@param[in]	len		Data length.
 *
 * 	@param[out]	ciphertxt	Result buffer for the cipher-text.
 *					Allocated by the library.
 *                                      Use bb5_dealloc_result() to deallocate.
 *
 *	@return		>=0		Success, size of the result buffer.
 *
 *	@return		<0		Generic error.
 *
*/
extern ssize_t bb5_fprot_encrypt(const uint8_t *msg,
				 size_t len,
				 uint8_t **ciphertxt);



/**
 * @brief Encrypt data.
 * @details Encrypts data with a device-specific key.
 * Do NOT use this, unless you really know know what you are doing
 *
 * 	@param[in]	msg		Data.
 *
 * 	@param[in]	len		Data length.
 *
 * 	@param[out]	ciphertxt	Result buffer for the cipher-text.
 *					Allocated by the library.
 *                                      Use bb5_dealloc_result() to deallocate.
 *
 *	@return		>=0		Success, size of the result buffer.
 *
 *	@return		<0		Generic error.
 *
*/
extern ssize_t bb5_fprot_encrypt_without_salt(const uint8_t *msg,
                                              size_t len,
                                              uint8_t **ciphertxt);

/**
 * @brief Decrypt data.
 * @details Decrypts data encrypted with bb5_fprot_encrypt() or
 * bb5_fprot_encrypt_without_salt().
 *
 * 	@param[in]	msg		Cipher-text.
 *
 * 	@param[in]	len		Cipher-text length.
 *
 * 	@param[out]	plaintxt	Result buffer for the plain-text.
 *					Allocated by the library.
 *                                      Use bb5_dealloc_result() to deallocate.
 *
 *	@return		>=0		Success, size of the result buffer.
 *
 *	@return		<0		Generic error.
 *
*/
extern ssize_t bb5_fprot_decrypt(const uint8_t *msg,
                                 size_t len,
                                 uint8_t **plaintxt);


/**
 * @brief Compute a MAC.
 * @details Computes a Message Authentication Code (MAC) of the given data.
 *
 * 	@param[in]	msg		Data.
 *
 * 	@param[in]	len		Data length.
 *
 * 	@param[out]	authcode	Result buffer for the MAC.
 *					Allocated by the library.
 *                                      Use bb5_dealloc_result() to deallocate.
 *
 *	@return		>=0		Success, size of the result buffer.
 *
 *	@return		<0		Generic error.
 *
*/
extern ssize_t bb5_fprot_mac(const uint8_t *msg,
                             size_t len,
                             uint8_t **authcode);




/**
 * @brief Validate RSA signature and compute a MAC.
 * @details Checks a given RSA signature, and if successfull computes a
 * Message Authentication Code (MAC) of the given data.
 *
 * 	@param[in]	msg		Data.
 *
 * 	@param[in]	len		Data length.
 *
 * 	@param[in]	rsa_len		RSA signature length.
 *
 * 	@param[in]	signature	RSA signature.
 *
 * 	@param[in]	pub_expo	RSA public exponent.
 *
 * 	@param[in]	pub_modul	RSA public modulus.
 *
 * 	@param[out]	authcode	Result buffer for the MAC.
 *					Allocated by the library.
 *                                      Use bb5_dealloc_result() to deallocate.
 *
 *	@return		>=0		Success, size of the result buffer.
 *	    		<0		Generic error.
 *
*/
extern ssize_t bb5_fprot_mac_rsa(const uint8_t *msg,
                                 size_t len,
                                 size_t rsa_len,
                                 const uint8_t *signature,
                                 uint32_t pub_expo,
                                 const uint8_t *pub_modul,
                                 uint8_t **authcode);

/**
 * @brief Checks a MAC.
 * @details Checks a given Message Authentication Code (MAC) of the
 * given data for a given application ID.
 *
 * 	@param[in]	prog_id		Application ID.
 *
 * 	@param[in]	msg		Data.
 *
 * 	@param[in]	len		Data length.
 *
 * 	@param[in]	authcode	Result buffer for the MAC.
 *					Allocated by the library.
 *                                      Use bb5_dealloc_result() to deallocate.
 *
 * 	@param[in]	authlen		Result buffer length.
 *
 *	@return		>=0		Success, information on the MAC.
 *                                      See enum bb5_fprot_mac_result for
 *                                      more information on this.
 *	    		<0		Generic error.
 *
*/
extern int bb5_fprot_check_mac(const uint8_t *prog_id,
                                   const uint8_t *msg,
                                   size_t len,
                                   const uint8_t *authcode,
                                   size_t authlen);

/******************************************************
               FPROT services header END
*******************************************************/


/******************************************************
              KERNEL MODE services header START
*******************************************************/

#define  KERNEL_MODE_UNSET        0
#define  KERNEL_MODE_TRUSTED      1
#define  KERNEL_MODE_ENOSW    2
#define  KERNEL_MODE_UNTRUSTED    3

/**
 * @brief KERNEL MODE service.
 * @details Retrieves kernel mode.
 *
 *	@return		0		Kernel mode unset.
 *	    		1		Trusted kernel.
 *	    		2		ENOSW kernel.
 *	    		3		Untrusted kernel.
 *	    		<0		Generic error.
 *
*/
extern int bb5_get_kernel_mode();

/******************************************************
                KERNEL MODE services header END
*******************************************************/



/******************************************************
       Multi-ASIC labelling services header START
*******************************************************/

typedef uint8_t bb5_label2_assk_mask_t[16];

/**
 * @brief Executes labelling challenge-response.
 * @details Executes challenge response authentication of the device in
 * multi-ASIC labelling. Used at the production line in labelling phase
 * and in RnD certificate generation.
 *
 *  @credential libbb5-secbins::SEE_TerminalResponce    Required for all use cases
 *
 * 	@param[in]	buf		The challenge buffer.
 *                                      Must not be NULL.
 *
 * 	@param[in]	len		Length of the challenge buffer.
 *
 * 	@param[in]	mask		Bitmask for ASSK generation.
 *                                      Must not be NULL.
 *
 * 	@param[in]	pubid		Public ID of the master ASIC.
 *                                      In case of master ASIC labelling (APE),
 *                                      the ID is not really used.
 *                                      Must not be NULL pointer.
 *
 * 	@param[in]	ic_index	IC index for the IC used.
 *
 * 	@param[out]	resp		Result buffer for the response.
 *                                      Allocated by the library.
 *                                      Use bb5_dealloc_result() to deallocate.
 *
 *	@return		>=0		Success, length of response buffer.
 *	    		<0		Generic error.
 *
*/
extern ssize_t bb5_label2_resp_create(  const uint8_t * buf,
                                	size_t len,
					const bb5_label2_assk_mask_t *mask,
                              		const bb5_publicid_t *pubid,
                                        uint8_t ic_index,
                                	uint8_t **resp);


/******************************************************
       Multi-ASIC labelling services header END
*******************************************************/


/******************************************************
            Public ID services header START
*******************************************************/


/**
 * @brief Retrieve public ID.
 * @details Retrieves the public ID of the device from secure environment.
 *
 * 	@param[out]	id		Result buffer for the ID.
 *                                      Allocated by the caller.
 *
 *	@return		0		Success.
 *	    		<0		Generic error.
 *
*/
extern int bb5_get_publicid(bb5_publicid_t *id);

/******************************************************
            Public ID services header END
*******************************************************/


/******************************************************
              RDC services header START
*******************************************************/

/**
 * @brief Decodes RDC bits.
 * @details Decodes the given RnD certificate bit masks into human
 * readable labels.
 *
 * 	@param[in]	hw_bits		HW bitmask
 *
 * 	@param[in]	sw_bits		SW bitmask
 *
 * 	@param[out]	hw_result	Result buffer for an array of zero-terminated
 *                                      labels for the decoded HW bits.
 *                                      Allocated by the library.
 *                                      Deallocation through bb5_dealloc_result();
 *
 * 	@param[out]	hw_count	Number of labels in the result buffer.
 *                                      Allocated by the caller.
 *
 * 	@param[out]	sw_result	Result buffer for an array of zero-terminated
 *                                      labels for the decoded SW bits.
 *                                      Allocated by the library.
 *                                      Deallocation through bb5_dealloc_result();
 *
 * 	@param[out]	sw_count	Number of labels in the result buffer.
 *                                      Allocated by the caller.
 *
 *	@return		0		Success.
 *
 *	@return		<0		Generic error.
 *
*/
extern int bb5_rdc_decode_bits(uint32_t hw_bits,
                               uint32_t sw_bits,
                               bb5_labelstr_t *hw_result[],
                               size_t *hw_count,
                               bb5_labelstr_t *sw_result[],
                               size_t *sw_count);


/**
 * @brief Verifies the RDC.
 * @details Verifies the integrity and authenticity of the RND
 * certificate
 *
 * 	@param[out]	buf		Certificate data buffer.
 *					If NULL, reads cert from CAL area
 *
 * 	@param[in]	len		Size of the data buffer
 *
 * 	@param[out]	hw_bits		Result buffer for the RDC HW bits.
 *                                      Allocated by the caller.
 *					If NULL, does not return bits.
 *
 * 	@param[out]	sw_bits		Result buffer for the RDC SW bits.
 *                                      Allocated by the caller.
 *					If NULL, does not return bits.
 *
 *	@return		0		Success.
 *
 *	@return		<0		Generic error.
 *
*/
extern int bb5_rdc_cert_verify( const uint8_t *buf,
                                size_t len,
                                uint32_t *hw_bits,
                                uint32_t *sw_bits);

/**
 * @brief Flashes RDC to CAL.
 * @details Flashes the RnD certificate to the CAL area.
 * Validates the integrity and authenticity of the certificate
 *
 *  @credential GRP::cal    Required for all use cases
 *
 * 	@param[out]	buf		Certificate data buffer
 *
 * 	@param[in]	len		Size of the data buffer
 *
 *	@return		0		Success.
 *	    		<0		Generic error.
 *
*/
extern int bb5_rdc_cert_flash(const uint8_t * buf, size_t len);

/**
 * @brief Erases RDC from CAL.
 * @details Erases the RnD Certificate from the CAL area.
 *
 *  @credential GRP::cal    Required for all use cases
 *
 *	@return		0	Success.
 *	    		<0	Generic error.
 *
*/
extern int bb5_rdc_cert_erase();

/**
 * @brief Reads RDC from CAL.
 * @details Reads the RnD certificate from the CAL area.
 *
 *  @credential GRP::cal    Required for all use cases
 *
 * 	@param[out]	buf		Result buffer for the certificate
 *                                      data. Allocated by the library.
 *
 * 	@param[out]	len		Size of the data buffer, set
 *                                      by the library.
 *
 *	@return		0		Success.
 *	    		<0		Generic error.
 *
*/
extern int bb5_rdc_cert_read(uint8_t **buf, size_t *len);

/******************************************************
              RDC services header END
*******************************************************/


/******************************************************
              RNG services header START
*******************************************************/


/**
 * @brief RNG service.
 * @details Retrieves a random number from the hardware accelerated
 * random number generator in the secure environment.
 *
 * 	@param[out]	buf		Result buffer for the random data.
 *
 * 	@param[in]	len		Size of the result buffer in bytes.
 *
 *	@return		0		Success.
 *	    		<0		Generic error.
 *
*/
extern int bb5_get_random(uint8_t *buf, size_t len);

/******************************************************
                RNG services header END
*******************************************************/



/******************************************************
            Debug Control services header START
*******************************************************/


/**
 * @brief Retrieve debug control.
 * @details Retrieves the debug control of the device from secure environment.
 *
 * 	@param[out]	dbg		Result buffer.
 *                                      Allocated by the caller.
 *
 *	@return		0		Success.
 *	    		<0		Generic error.
 *
*/
extern int bb5_get_debug_control(bb5_debug_control_t *dbg);

/******************************************************
            Debug Control services header END
*******************************************************/
/******************************************************
            Fast IMEI services header START
*******************************************************/

/**
 * @brief Retrieve IMEI
 * @details Retrieves the IMEI of the device from secure environment.
 *
 * 	@param[out]	dbg		Result buffer.
 *                                      Allocated by the caller.
 *
 *	@return		0		Success.
 *	    		<0		Generic error.
 *
*/
extern int bb5_fast_imei_read(bb5_imei_bcd_t *imei);

/******************************************************
            Fast IMEI read services header END
*******************************************************/


/******************************************************
             ROT13 services header START
*******************************************************/

/**
 * @brief ROT13 test.
 * @details Does a ROT13 encoding on given buffer using ROT13 PA.
 *
 * 	@param[in,out]	buf		Buffer to rotate.
 *
 * 	@param[in]	len		Size of buffer in bytes.
 *
 *	@return		0		Success.
 *	    		<0		Generic error.
 *
*/
extern int bb5_rot13(uint8_t *buf, size_t len);

/******************************************************
              ROT13 services header END
*******************************************************/


/******************************************************
         Super Dongle services header START
*******************************************************/

/** SuperDongle type */
typedef uint8_t bb5_sd_type_t[2];
 /** SuperDongle smartcard serial number */
typedef uint8_t _bb5_sd_serial_t[6];
/** Latest date when the SuperDongle is still valid */
typedef uint8_t _bb5_sd_validthru_t[3];
/** Session ID 1 type, used for generating session key 1*/
typedef uint8_t _bb5_sd_sessionid1_t[16];
/** Session ID 2 type, used for generating session key 2*/
typedef uint8_t _bb5_sd_sessionid2_t[4];
/** Array of mobile terminal specific keys */
typedef uint8_t _bb5_sd_mtkeys_t[8][10];

typedef uint8_t bb5_sd_hmac_t[8];


/** Supported status of SD connections */
enum bb5_sd_status {
	BB5_SD_STATUS_NONE = 0,
	BB5_SD_STATUS_INITIALIZED,
	BB5_SD_STATUS_AUTHENTICATED,
	BB5_SD_STATUS_VERIFIED,
};

/** Supported values for SD key import status */
enum bb5_sd_keys_status {
	BB5_SD_KEY_NOT_IMPORTED = 0,
	BB5_SD_KEYS_IMPORTED = 1,
};

/** Status of authenticated SD connection when timeout is triggered */
enum bb5_sd_timeout_reason {
        /** Authentication process has been completed */
        BB5_SD_TIMEOUT_COMPLETE_CONN = 1,
        /** Authentication process is not yet complete */
        BB5_SD_TIMEOUT_INCOMPLETE_CONN = 2,
};

/** Encrypted block of input data for SD authentication phase 3/3 */
struct bb5_sd_verify_block
{
    uint8_t message_id;
    uint8_t result;
    uint8_t reserved[14];
    bb5_sd_hmac_t hmac;
};

/** Encrypted block of input data for SD authentication phase 2/3 */
struct bb5_sd_auth_block
{
    uint8_t message_id; /**< Message ID of the authentication process */
    bb5_sd_type_t sd_type;
    _bb5_sd_serial_t sc_serial_num;
    _bb5_sd_validthru_t valid_thru;
    _bb5_sd_sessionid2_t session_id2;
    bb5_sd_hmac_t hmac;
};

/** Encrypted block of input data for SD authentication phase 1/3 */
struct bb5_sd_prod_block
{
    _bb5_sd_mtkeys_t mt_keys;
    uint8_t key_index;
    uint8_t padding[7]; /* AES in 16-byte blocks */
    bb5_sd_hmac_t hmac;
};

/** SuperDongle identification information */
struct bb5_sd_id
{
    bb5_sd_type_t sd_type;
    _bb5_sd_serial_t sc_serial_num;
};

/** SD authentication success response */
struct _bb5_sd_res_auth_ok
{
    uint8_t message_id;
    uint8_t response;
    uint8_t key_index;
    uint8_t reserved[13];
    bb5_sd_hmac_t hmac;
};


/** SD authentication failure response */
struct _bb5_sd_res_auth_kill
{
    uint8_t message_id;
    uint8_t response;
    uint8_t reserved[14];
    bb5_sd_hmac_t hmac;
};

/** Information on the authenticated SuperDongle */
struct _bb5_sd_res_auth_info
{
    bb5_sd_type_t sd_type;
    _bb5_sd_serial_t sc_serial_num;
    _bb5_sd_validthru_t valid_thru;
};

/** Result buffer type for the SD authentication phase 2/3 */
struct bb5_sd_res_auth
{
    struct _bb5_sd_res_auth_info sd_info;
    struct _bb5_sd_res_auth_ok ok_msg;
    struct _bb5_sd_res_auth_kill kill_msg;
};

/** Result buffer type for the SD authentication phase 1/3 */
struct bb5_sd_res_auth_init
{
    _bb5_sd_sessionid1_t session_id1;
    bb5_publicid_t public_id;
    uint8_t algorithm;
};

/**
 * @brief Flashes SD data to CAL.
 * @details Writes mobile-terminal specific keys to secure storage.
 * Used in labelling phase at the production line.
 * Does integrity and authenticity check for the data.
 *
 * NOTE: requires root privileges because of dependency to libcal!
 *
 *  @credential libbb5-secbins::SEE_SuperDongleWrite    Required for all use cases
 *  @credential GRP::cal    Required for all use cases
 *
 *
 * 	@param[in]	block		SD data.
 *
 *	@return		0		Success.
 *	    		<0		Generic error.
 *
*/
extern int bb5_sd_data_flash(const struct bb5_sd_prod_block *block);

/**
 * @brief SD production init.
 * @details Executes SuperDongle production initialization phase.
 * Writes mobile-terminal specific keys to secure storage.
 * Used in labelling phase at the production line.
 *
 * 	@param[in]	block		Parameters for secure side.
 *
 *	@return		0		Success.
 *	    		<0		Generic error.
 *
*/
extern int bb5_sd_prod_init(const struct bb5_sd_prod_block *block) __attribute__ ((deprecated));



/**
 * @brief SD authentication, phase 1/3.
 * @details Executes SuperDongle authentication initialization phase.
 * Initiates a SuperDongle session for authentication.
 *
 *  @credential     libbb5-secbins::SEE_SuperDongleOperation    Required for all use cases
 *  @credential GRP::cal    Required for all use cases
 *
 * 	@param[out]	init_resp	Result buffer for secure side response.
 *                                      Must be allocated by the caller.
 *
 *	@return		0		Success.
 *	    		<0		Generic error.
 *
*/
extern int bb5_sd_auth_init(struct bb5_sd_res_auth_init *init_resp);


/**
 * @brief SD authentication, phase 2/3.
 * @details Authenticates the SuperDongle smart card to the device.
 * SD session must be initialized with bb5_sd_auth_init() before
 * calling this service.
 *
 *  @credential     libbb5-secbins::SEE_SuperDongleOperation    Required for all use cases
 *  @credential GRP::cal    Required for all use cases
 *
 * 	@param[in]	block		Parameters for secure side.
 *
 * 	@param[out]	auth_resp	Result buffer for the secure side
 *                                      response. Must be allocated by the
 *                                      caller.
 *
 *	@return		0		Success.
 *	    		<0		Generic error.
 *
*/
extern int bb5_sd_auth(const struct bb5_sd_auth_block *block,
                           struct bb5_sd_res_auth *auth_resp);


/**
 * @brief SD authentication, phase 3/3.
 * @details Verifies the authentication of SuperDongle smart card to
 * the device. SD session must be initialized with bb5_sd_auth_init()
 * and bb5_sd_auth() before calling this service.
 *
 *  @credential libbb5-secbins::SEE_SuperDongleOperation    Required for all use cases
 *
 * 	@param[in]	block		Parameters for secure side.
 *
 *	@return		0		Success.
 *	    		<0		Generic error.
 *
*/
extern int bb5_sd_verify(const struct bb5_sd_verify_block *block);


/**
 * @brief SD authentication timeout.
 * @details Ends the authenticated connection between SuperDongle
 * smartcard and the device. Executed after a specific timeout.
 *
 *  @credential libbb5-secbins::SEE_SuperDongleOperation    Required for all use cases
 *
 * 	@param[in]	reason		The State of the connection when
 *                                      the timeout was triggered.
 *
 *	@return		0		Success.
 *	    		<0		Generic error.
 *
*/
extern int bb5_sd_auth_timeout(enum bb5_sd_timeout_reason reason);


/**
 * @brief Read SD IDs.
 * @details Retrieves a list of recently used SuperDongle IDs.
 *
 *  @credential libbb5-secbins::SEE_SuperDongleOperation    Required for all use cases
 *  @credential GRP::cal    Required for all use cases
 *
 *
 * 	@param[out]	res_list	The result buffer for an array of SD
 *                                      IDs. The library will allocate the
 *                                      buffer.
 *                                      Use bb5_dealloc_result() to deallocate.
 *
 *	@return		>=0		Success, number of IDs retrieved
 *	    		<0		Generic error.
 *
*/
extern ssize_t bb5_sd_id_list_read(struct bb5_sd_id **res_list);


/**
 * @brief Clear used SD IDs.
 * @details Clears the list of recently sed SD IDs.
 *
 *  @credential libbb5-secbins::SEE_SuperDongleOperation    Required for all use cases
 *  @credential GRP::cal    Required for all use cases
 *
 *	@return		0		Success
 *	    		<0		Generic error.
 *
*/
extern int bb5_sd_id_list_clear(void);


/**
 * @brief Read status of SD keys.
 * @details Retrieves the import status of the Super Dongle mobile
 * terminal specific keys.
 *
 *  @credential libbb5-secbins::SEE_SuperDongleOperation    Required for all use cases
 *  @credential GRP::cal    Required for all use cases
 *
 * 	@param[out]	status	        The result buffer for the
 *                                      import status of the keys.
 *                                      The buffer must be allocated by the
 *                                      caller
 *
 *	@return		0		Success
 *	    		<0		Generic error.
 *
*/
extern int bb5_sd_keys_status_get(enum bb5_sd_keys_status *status);


/**
 * @brief Read SD authentication status.
 * @details Retrieves the authentication status of the SuperDongle
 * session and the used SuperDongle type.
 *
 *  @credential libbb5-secbins::SEE_SuperDongleOperation    Required for all use cases
 *  @credential GRP::cal    Required for all use cases
 *
 *
 * 	@param[out]	status	        The result buffer for the
 *                                      authentication status of the SD session
 *                                      The buffer must be allocated by the
 *                                      caller
 *
 * 	@param[out]	type	        The result buffer for the type of SD.
 *                                      The buffer must be allocated by the
 *                                      caller
 *
 *	@return		0		Success
 *	    		<0		Generic error.
 *
*/
extern int bb5_sd_status_get(enum bb5_sd_status *status, bb5_sd_type_t *type);


/******************************************************
            Super Dongle services header END
*******************************************************/


/******************************************************
         Secure storage services header START
*******************************************************/


/**
 * @brief Loads secure storage.
 * @details Reads secure storage block from CAL and feeds it to the device.
 *
 *  @credential GRP::cal    Required for all use cases
 *  @credential libbb5-secbins::SEE_SecStorageMaintenance    Required for all use cases
 *
 *	@return		0		Success.
 *	    		<0		Generic error.
 *
*/
extern int bb5_load_secs(void);

/**
 * @brief Saves secure storage.
 * @details Reads secure storage buffer from kernel and writes it to CAL.
 *
 *  @credential GRP::cal    Required for all use cases
 *  @credential libbb5-secbins::SEE_SecStorageMaintenance    Required for all use cases
 *
 *	@return		0		Success.
 *	    		<0		Generic error.
 *
*/
extern int bb5_save_secs(void);

/******************************************************
          Secure storage services header END
*******************************************************/


/*
* Public API for the Simlock PA
*/

struct bb5_sl_data_server {
    uint8_t   version;
    uint8_t   active_index;
    uint8_t   reserved[4];
    uint8_t   verifyresult0[20];
    uint8_t   verifyresult1[20];
    uint8_t   verifyresult2[20];
    uint8_t   verifyresult3[20];
    uint8_t   verifyresult4[20];
    uint8_t   verifyresult5[20];
    uint8_t   verifyresult6[20];
    uint8_t   verifyresult7[20];
        uint8_t   public_id[20];
    uint8_t   signature[128];
    uint16_t  SL_data_len;
    uint8_t   SL_data[0];
};

/**
 * @brief Verify data and store it if there are no errors
 * @details Verify data and store it if there are no errors
 * NOTE: Requires tokens SEE_SimLock3Write
 * for all use cases!
 *
 *
 * @credential  libbb5-secbins::SEE_SimLock3Write    Required for all use cases
 * @credential  GRP::cal                             Required for all use cases
 *
 * @param[in]	sl_data_server  SIMlock certificate from security server
 *
 * @return      0 on success, or -1 on failure.
 *
*/
extern int bb5_sl_data_flash( const struct bb5_sl_data_server *sl_data_server);

/**
 * @brief Verify unlock code and store it if there are no errors
 * @details Verify unlock code and store it if there are no errors
 * NOTE: Requires tokens SEE_SimLock3Operation
 * for all use cases!
 *
 *
 * @credential  libbb5-secbins::SEE_SimLock3Operation    Required for all use cases
 * @credential  GRP::cal                                 Required for all use cases
 *
 * @param[in]	level   SIMLock level
 * @param[in]	code*   Pointer to unlock code
 * @param[in]	len     Length of unlock code
 *
 * @return  0 on success, or <0 on failure.
*/
extern int bb5_sl_unlock(int level, const char* code, int len);

/**
 * @brief Returns status of simlock password
 * @details Returns status of simlock password
 * NOTE: Requires tokens SEE_SimLock3Operation
 * for all use cases!
 *
 * @credential     libbb5-secbins::SEE_SimLock3Operation    Required for all use cases
 * @credential     GRP::cal                                 Required for all use cases
 *
 * @param[in]	level   SIMLock level
 *
 * @return  0 on success password status is ok
 *          1 if no password for given level is found
 *          <0 on failure.
 *
*/
extern int bb5_sl_get_status(int level);

/**
 * @brief Returns status of all simlock passwords
 * @details Returns status of all simlock passwords
 * NOTE: Requires tokens SEE_SimLock3Operation
 * for all use cases!
 *
 *
 * @credential     libbb5-secbins::SEE_SimLock3Operation    Required for all use cases
 * @credential     GRP::cal                                 Required for all use cases
 *
 * @return  0 on success if any correct password is found
 *          1 if no correct password is found
 *          <0 on failure.
*/
extern int bb5_sl_get_status_all();

/**
 * @brief Returns status simlock
 * @details Returns status simlock
 * NOTE: Requires tokens SEE_SimLock3Operation
 * for all use cases!
 *
 *
 * @credential     libbb5-secbins::SEE_SimLock3Operation    Required for all use cases
 * @credential     GRP::cal                                 Required for all use cases
 *
 * @return  0 if simlock is locked (either allways locked or locked with password and no password inserted)
 *          1 if simlock is open   (either allways open or locked with password and correct password inserted)
 *          <0 on failure.
*/
extern int bb5_sl_is_locked();

/**
 * @brief Returns simlock certificate ape image lock bits
 * @details Returns simlock certificate ape image lock bits
 * NOTE: Requires tokens SEE_SimLock3Operation
 * for all use cases!
 *
 * @credential     libbb5-secbins::SEE_SimLock3Operation    Required for all use cases
 * @credential     GRP::cal                                 Required for all use cases
 *
 * @return  0 if imagelock is allways locked
 *          2 if open lock allow with password
 *          3 if imagelock is allways open
 *          >3 on error
 *
*/
extern int bb5_sl_get_lock_bits();


/**
* @brief Clear unlock codes from CAL area
* @details Clear unlock codes from CAL area
* @return   0 on success, or -1 on failure.
*/
extern int bb5_sl_reset();

extern int bb5_sl_test_only();


/******************************************************
           SW certificate services header START
*******************************************************/

struct _bb5_signer_info {
        uint32_t server_id;
        uint32_t time_stamp;
        uint32_t serial_number;
        uint32_t user_id;
} __attribute__ ((packed));

struct _bb5_sec_rom_hash_descriptor {
        uint32_t start;
        uint32_t end;
        bb5_hash_sha1_t  data;
} __attribute__ ((packed));

struct _bb5_omap_speedup {
        uint32_t magic;
        uint32_t mask;
        uint32_t data[32][2];
        uint32_t poll;
        uint32_t spare;
} __attribute__ ((packed)); /* 272 bytes */


struct bb5_common_header {
        uint32_t magic;
        uint32_t key_index;
        struct _bb5_signer_info signer;
        uint32_t signed_header_size;
        struct _bb5_sec_rom_hash_descriptor hash;
        struct _bb5_omap_speedup speedup;
        uint32_t startup;
        uint32_t watchdog_param; /* 14C */
        uint16_t signed_key_type;
        uint16_t ppa_type;
        uint32_t version; /* 154 */
        uint32_t submagic;
        uint32_t check_interval; /* 15C */
        uint32_t check_flags;
        uint8_t data3[492]; /* 164 */
        uint8_t head_signature[128];
} __attribute__ ((packed)); /* sizeof(bb5_common_header) == 976
                             * (0x3D0), +align = 0x600 */

struct bb5_sw_cert_subimage {
        char name[12];
        bb5_hash_sha1_t sha;
} __attribute__ ((packed));

struct bb5_sw_cert_v1 {
        struct bb5_common_header bb5_header;
        uint32_t version;
        uint32_t timestamp;
        uint32_t entries_v1;
        struct bb5_sw_cert_subimage images[];
} __attribute__ ((packed));

struct bb5_sw_cert_v2 {
        struct bb5_common_header bb5_header;
        uint32_t version;
        uint32_t timestamp;
        char sw_version[128];
        uint32_t entries;
        struct bb5_sw_cert_subimage images[];
} __attribute__ ((packed));

#define SEC_FLAGS_TRUSTED	0x01
#define SEC_FLAGS_FLASHING	0x02
struct bb5_secinfo_config {
	uint32_t sec_flags;
	uint32_t mode_count;
};

#define SEC_FLAGS_FIRST_BOOT	0x01
#define SEC_FLAGS_LOCKED	0x02
struct bb5_secinfo_user {
	int	sec_flags;
};

/*
 * Verifies BB5 signature
 */
extern int bb5_verify(const void *img, size_t len);


/**
 * @brief Verifies SWCert
 * @details Verifies the integrity and authenticity of the SW
 * certificate
 *
 * 	@param[in]	data	Pointer to header start
 *
 * 	@param[in]	len		Length of header+data
 *
 *	@return		0		Success.
 *	    		-2		Invalid length.
 *	    		<0		Generic error.
 *
*/
extern int bb5_swcert_verify(const struct bb5_sw_cert_v2 *cert, size_t len);


extern int bb5_swcert_flash(const struct bb5_sw_cert_v2 *cert, size_t len);
extern int bb5_swcert_read(struct bb5_sw_cert_v2 **cert, size_t *len);
extern int bb5_secinfo_config_write(const struct bb5_secinfo_config *si);
extern int bb5_secinfo_config_read(struct bb5_secinfo_config **si);
extern int bb5_secinfo_user_write(const struct bb5_secinfo_user *si);
extern int bb5_secinfo_user_read(struct bb5_secinfo_user **si);

/******************************************************
           SW certificate services header END
*******************************************************/




#ifdef __cplusplus
}
#endif

#endif /* LIBBB5_H */
